package com.laguerre.kan;

public final class Color {
	public static final Color BLACK = getNewBlack();
	public static final Color GREY = getNewGrey();
	public static final Color WHITE = getNewWhite();
	public static final Color RED = getNewRed();
	public static final Color GREEN = getNewGreen();
	public static final Color BLUE = getNewBlue();

	private final float[] color = { 0f, 0f, 0f, 1f };

	public Color() {
		// Does nothing, keeps the default color.
	}

	public Color(float r, float g, float b, float a) {
		setValues(r, g, b, a);
	}

	public Color(Color c) {
		setValues(c);
	}

	public void setValues(float r, float g, float b, float a) {
		this.color[0] = r;
		this.color[1] = g;
		this.color[2] = b;
		this.color[3] = a;
	}

	public void setValues(Color c) {
		if (c != this) {
			System.arraycopy(c.color, 0, this.color, 0, 4);
		}
	}

	public String toString() {
		return "(" + this.color[0] + ", " +
				this.color[1] + ", " +
				this.color[2] + ", " +
				this.color[3] + ")";
	}

	public void getComponents(float[] result) {
		System.arraycopy(this.color, 0, result, 0, 4);
	}

	/**
	 * Gets a direct reference to the array of <code>float</code> components of
	 * this <code>Color</code>. This method must never been used to modify this
	 * color but only to avoid performance penalty due to data copy.
	 *
	 * @return A reference to the array of color components.
	 */
	public float[] getSharedComponents() {
		return this.color;
	}

	public float getR() {
		return this.color[0];
	}

	public float getG() {
		return this.color[1];
	}

	public float getB() {
		return this.color[2];
	}

	public float getA() {
		return this.color[3];
	}

	public static Color getNewBlack() {
		return new Color(0f, 0f, 0f, 1f);
	}

	public static Color getNewGrey() {
		return new Color(0.5f, 0.5f, 0.5f, 1f);
	}

	public static Color getNewWhite() {
		return new Color(1f, 1f, 1f, 1f);
	}

	public static Color getNewRed() {
		return new Color(1f, 0f, 0f, 1f);
	}

	public static Color getNewGreen() {
		return new Color(0f, 1f, 0f, 1f);
	}

	public static Color getNewBlue() {
		return new Color(0f, 0f, 1f, 1f);
	}
}
