package com.laguerre.kan;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import android.content.Context;
import android.opengl.GLES20;
import android.util.Log;

public final class Utils {
	/**
	 * Represents a bind to a framebuffer that has not been created.
	 */
	public static class WrongFrameBufferHandleException extends RuntimeException {
		private static final long serialVersionUID = 9066562408218138881L;

		public WrongFrameBufferHandleException(String msg) {
			super(msg);
		}
	}

	public static class FrameBufferIncompleteException extends RuntimeException {
		private static final long serialVersionUID = -8348255784559121128L;

		public FrameBufferIncompleteException(String msg) {
			super(msg);
		}
	}

	private static Context currentContext;

	public static void setContext(Context context) {
		currentContext = context;
	}

	public static Context getContext() {
		return currentContext;
	}

	public static String readTextFileFromRawResource(final Context context,
			final int resourceId) {
		final InputStream inputStream = context.getResources().openRawResource(
				resourceId);
		final InputStreamReader inputStreamReader = new InputStreamReader(
				inputStream);
		final BufferedReader bufferedReader = new BufferedReader(inputStreamReader, 8192);

		final StringBuilder body = new StringBuilder();
		try {
			String nextLine = null;
			while ((nextLine = bufferedReader.readLine()) != null) {
				body.append(nextLine);
				body.append('\n');
			}
		} catch (IOException e) {
			return null;
		}
		return body.toString();
	}

	public static void checkGlError(String glOperation) {
		checkGlError("kan", glOperation);
	}

    /**
     * Utility method for debugging OpenGL calls. Provide the name of the call
     * just after making it:
     *
     * <pre>
     * mColorHandle = GLES20.glGetUniformLocation(mProgram, "vColor");
     * MyGLRenderer.checkGlError("glGetUniformLocation");</pre>
     *
     * If the operation is not successful, the check throws an error.
     *
     * @param glOperation - Name of the OpenGL call to check.
     * @param tag - A tag to recognize logs.
     */
    public static void checkGlError(String tag, String glOperation) {
        int error;
        while ((error = GLES20.glGetError()) != GLES20.GL_NO_ERROR) {
            Log.e(tag, glOperation + ": glError " + error);
            throw new RuntimeException(glOperation + ": glError " + error);
        }
    }
}
