package com.laguerre.mytest;

import java.util.ArrayList;

import com.laguerre.kan.Behaviour;
import com.laguerre.kan.SceneObject;
import com.laguerre.kan.Vector3;

public class MetaballModel extends Behaviour {
	private static final class MetaballInfo {
		public final Vector3 center = Vector3.getNewZeroVec();
		public float radius;

		public MetaballInfo(Vector3 center, float radius) {
			this.center.setValues(center);
			this.radius = radius;
		}
	}

	private final ArrayList<MetaballInfo> metaballInfos = new ArrayList<MetaballInfo>();

	public final float fluidDensity(Vector3 point, float scale, float radius) {
		float accu = 0f;
		for (MetaballInfo ball : this.metaballInfos) {
			accu += weight(Vector3.minus(point, ball.center), radius);
		}
		return scale * accu;
	}

	private float weight(Vector3 center, float radius) {
		final float rightElm = (radius * radius) - center.getSquareNorm();
		if (rightElm > 0) {
			return (315f / (64f * (float) Math.PI * (float) Math.pow(radius, 9.f))) * rightElm * rightElm * rightElm;
		} else {
			return 0f;
		}
	}
}
