package com.laguerre.mytest;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.ShortBuffer;

import com.laguerre.kan.ManualMesh;

public class SimpleQuad extends ManualMesh {
	private static final float[] quadVertices = {
		-1.0f,  1.0f, 0.0f, 1.0f,   // top left
		-1.0f, -1.0f, 0.0f, 1.0f,   // bottom left
		 1.0f, -1.0f, 0.0f, 1.0f,   // bottom right
		 1.0f,  1.0f, 0.0f, 1.0f    // top right
	};

    private static final float[] normalCoords = {
        0.0f, 0.0f, 1.0f,   // top left
        0.0f, 0.0f, 1.0f,   // bottom left
        0.0f, 0.0f, 1.0f,   // bottom right
        0.0f, 0.0f, 1.0f    // top right
    };

	private static final float[] quadUV = {
		0.0f, 1.0f, // top left
		0.0f, 0.0f, // bottom left
		1.0f, 0.0f, // bottom right
		1.0f, 1.0f  // top right
	};

	private static short[] drawList = {
		0, 1, 2, 0, 2, 3
	};

	private FloatBuffer vertexBuffer;
	private final int vertexStride = 4 * Float.SIZE / 8;
	private FloatBuffer normalBuffer;
    private final int normalStride = 3 * Float.SIZE / 8;
	private FloatBuffer uvBuffer;
	private final int uvStride = 2 * Float.SIZE / 8;
	private ShortBuffer drawListBuffer;

    @Override
    public int getNbOfSubMeshes() {
    	return 1;
    }

	@Override
	public void build() {
    	ByteBuffer bb = ByteBuffer.allocateDirect(quadVertices.length * Float.SIZE / 8);
    	bb.order(ByteOrder.nativeOrder());
    	this.vertexBuffer = bb.asFloatBuffer();
    	this.vertexBuffer.put(quadVertices);
    	this.vertexBuffer.position(0);

        bb = ByteBuffer.allocateDirect(normalCoords.length * Float.SIZE / 8);
        bb.order(ByteOrder.nativeOrder());
        normalBuffer = bb.asFloatBuffer();
        normalBuffer.put(normalCoords);
        normalBuffer.position(0);

    	bb = ByteBuffer.allocateDirect(quadUV.length * Float.SIZE / 8);
    	bb.order(ByteOrder.nativeOrder());
    	this.uvBuffer = bb.asFloatBuffer();
    	this.uvBuffer.put(quadUV);
    	this.uvBuffer.position(0);

    	bb = ByteBuffer.allocateDirect(drawList.length * Short.SIZE / 8);
    	bb.order(ByteOrder.nativeOrder());
    	this.drawListBuffer = bb.asShortBuffer();
    	this.drawListBuffer.put(drawList);
    	this.drawListBuffer.position(0);
	}

	@Override
	public int getVertexCount(int subMeshId) {
		return 4;
	}

	@Override
	public int getVertexStride(int subMeshId) {
		return vertexStride;
	}

	@Override
	public FloatBuffer getSharedVertexBuffer(int subMeshId) {
		return this.vertexBuffer;
	}

	@Override
	public int getNormalStride(int subMeshId) {
		return this.normalStride;
	}

	@Override
	public FloatBuffer getSharedNormalBuffer(int subMeshId) {
		return this.normalBuffer;
	}

	@Override
	public int getSharedTexCoordStride(int subMeshId, int textureId) {
		return uvStride;
	}

	@Override
	public FloatBuffer getSharedTexCoordBuffer(int subMeshId, int textureId) {
		return this.uvBuffer;
	}

	@Override
	public int getDrawListCount(int subMeshId) {
		return drawList.length;
	}

	@Override
	public ShortBuffer getSharedDrawListBuffer(int subMeshId) {
		return this.drawListBuffer;
	}
}
